#!/usr/bin/python
# -*- coding: utf-8 -*-

# attention: pour les accents, ce fichier doit être impérativement
#            lu et modifié sous éditeur avec encodage UTF-8

################################################################
######                                                    ######
######          Module additionnel: conversions.py            ######
######                                                    ######
######            accompagnant la CALCULEXT               ######
######      [Calculatrice Extensible Version 1.40]        ######
######                                                    ######
################################################################

# Copyright (C) 2008 Jean-Paul Vidal dit "Tyrtamos" — Tous droits réservés.
#   Logiciel distribué sur le site <http://linux.jpvweb.com>
#
#   Ce programme est un logiciel libre. Vous pouvez le redistribuer ou le
#   modifier suivant les termes de la "GNU General Public License" version 3,
#   telle que publiée par la Free Software Foundation.
#
#   Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
#   AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
#   ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
#   Publique GNU pour plus de détails.
#
#   Ce programme doit être distribué avec une copie de la
#   License Générale Publique GNU (fichier license.html).
#   Si ce n’est pas le cas, consultez: <http://www.gnu.org/licenses/gpl.html>.

##################################################################
# Attention: configuration volontairement inhabituelle pour python
# les divisions entre 2 entiers seront décimales et pas entières!
# par exemple: 10/3 donnera "3.33333333333333" et pas "3"
# (les divisions entières seront désormais faites avec "//")
#
from __future__ import division

##################################################################
# Importation des modules système
#

##################################################################
# Importation des modules nécessaires aux calculs du présent module
#
from math import *
from copy import *
from decimal import *

from sys import modules
pmain = modules['__main__']

###############################################################################
# dit si une chaîne binaire est valide
#  si L est donnée (>0), C doit être de longueur L
#  C doit être une chaîne non vide ne contenant que des "0" et des "1"
#
def okbin(C, L=0):
    """okbin(C, L=0): dit si une chaîne binaire est valide. Renvoie True ou False"""
    if not pmain.estchaine(C):
        return False
    lg=len(C)
    if lg==0:
        return False
    if not pmain.estentier(L):
        return False
    if (L>0) and (lg!=L):
        return False
    if (C.count("0")+C.count("1"))!=lg:
        return False
    return True

###############################################################################
# Inversion des bits d'une chaîne binaire de longueur quelconque
#
def invbin(C):
    """invbin(C): inversion des bits d'une chaîne binaire de longueur quelconque"""
    if not okbin(C):
        raise ValueError ("erreur invbin(C): mauvaise chaîne binaire C")
    ch=''
    for i in xrange(0,len(C)):
        if C[i]=='0':
            ch+='1'
        else:
            ch+='0'
    return ch

###############################################################################
# Complément à 2 d'une chaîne binaire de longueur quelconque
#
def compbin2(C):
    """compbin2(C): complément à 2 d'une chaîne binaire de longueur quelconque"""
    # Vérif des données
    if not okbin(C):
        raise ValueError ("erreur compbin2(C): mauvaise chaîne binaire C")
    # Calcul
    return dec2bin(bin2dec(invbin(C))+1)

###############################################################################
# Conversion nombre entier positif -> chaîne binaire (si B>0, complète à gauche par des zéros)
#
def dec2bin(d,nb=0):
    """dec2bin(d,nb=0): conversion nombre entier positif ou nul -> chaîne binaire (si nb>0, complète à gauche par des zéros)"""
    # Vérif des données
    if (not pmain.estentier(d)) or (d<0) or (not pmain.estentier(nb)) or (nb<0):
        raise ValueError ("erreur dec2bin(d,nb=0): d et nb doivent être des entiers positifs ou nuls")
    if (nb>0) and (d>=(1<<nb)):
            raise ValueError ("erreur dec2bin(d,nb=0): d trop grand pour le nb donné")
    # Calcul
    if d==0:
        b="0"
    else:
        b=""
        while d!=0:
            b="01"[d&1]+b
            d=d>>1
    return b.zfill(nb)

###############################################################################
# Conversion nombre entier signé d -> chaîne binaire pour un mot de nb bits (0 par défaut)
#  Un nombre négatif est représenté par le complément à 2 de sa valeur absolue
#
def dec2bins(d,nb=0):
    """dec2bins(d,nb=0): conversion nombre entier signé d -> chaîne binaire pour un mot de nb bits (=0 par defaut) """
    # vérification des données
    if (not pmain.estentier(d)):
        raise ValueError ("erreur dec2bins(d,nb=0): d doit être un nombre entier")
    if (not pmain.estentier(nb)) or (nb<0):
        raise ValueError ("erreur dec2bins(d,nb=0): nb doit être un nombre entier positif ou nul")
    # calcul de la taille mini du mot permettant de représenter la valeur signée d (signe compris)
    n=1
    if d>=0:
        x=d
    else:
        x=-d-1
    while (1<<n)<=x:
        n+=1
    n+=1
    # valeur de nb à prendre pour la suite des calculs
    if (nb==0) or (nb<n):
        nb=n
    if nb<n:
        raise ValueError ("erreur dec2bins(d,nb=0): nb est trop faible pour le d donné")
    # calculs
    if d>=0:
        # on renvoie la chaîne normale, complétée par des "0" à gauche et par le bit de signe = "0"
        b='0' + dec2bin(d,nb-1)
    else:
        # on renvoie la chaîne signée, complétée par des "1" à gauche et par le bit de signe "1"
        b='1' + dec2bin((1<<(nb-1))+d,(nb-1))
    return b

###############################################################################
# Conversion chaîne binaire de longueur quelconque -> nombre entier positif
#
def bin2dec(C):
    """bin2dec(C): Conversion chaîne binaire de longueur quelconque -> nombre entier positif"""
    # vérif des données
    if not okbin(C):
        raise ValueError ("erreur bin2dec(C): mauvaise chaîne binaire C")
    # Calcul
    return int(C,2)

###############################################################################
# Conversion chaîne binaire de longueur B bits -> nombre entier signé
#
def bin2decs(C):
    """bin2decs(C): Conversion chaîne binaire signée de longueur quelconque -> nombre entier signé"""
    # vérif des données
    if not okbin(C):
        raise ValueError ("erreur bin2decs(C): mauvaise chaîne binaire C")
    # Calcul
    if C[0]=="0":
        # le 1er chiffre est un '0' => le résultat sera positif
        return int(C,2)
    else:
        # le 1er chiffre est un '1' => le résultat sera négatif
        return int(C,2)-(1<<len(C))

###############################################################################
# Conversion degrés décimaux -> liste: [degrés, minutes, secondes]
#
def deg2dms(dd):
    """deg2dms(dd): conversion degrés décimaux dd -> liste [degrés, minutes, secondes]"""
    # vérif des données
    if (not pmain.estnombre(dd)):
        raise ValueError ("erreur deg2dms(dd): dd doit être un nombre")
    # Calcul
    L=[]
    L.append(int(dd))
    L.append(int((dd-L[0])*60))
    L.append((((dd-L[0])*60)-L[1])*60)
    return L

###############################################################################
# Conversion degrés sous la forme d'une liste [degrés, minutes, secondes] -> en degrés décimaux
#
def dms2deg(L):
    """dms2deg(L): conversion degrés sous la forme d'une liste [degrés, minutes, secondes] -> en degrés décimaux"""
    # vérif des données
    if (not pmain.estliste(L)) or (len(L)!=3) or (not pmain.estnombre(L[0])) or (not pmain.estnombre(L[1])) or (not pmain.estnombre(L[2])):
        raise ValueError ("erreur dms2deg(L): L doit être une liste de 3 nombres")
    # Calcul
    return L[0] + (L[1]/60) + (L[2]/3600)

###############################################################################
# Convertit en chaîne hexadécimale les mots de passe donnés en texte clair
#  application: pour les mots de passe utilisés en wifi
#
def mdp2hex(ch):
    """mdp2hex(ch): Convertit en chaîne hexadécimale les mots de passe donnés en texte clair"""
    if not pmain.estchaine(ch):
        raise ValueError ("erreur mdp2hex(ch): ch doit être une chaîne de caractères")
    h=""
    for i in range(0,len(ch)):
        h+=str(hex(ord(ch[i])))[2:]
    return h

###############################################################################
# Convertit en texte clair les mots de passe donnés en chaîne hexadécimale
#  application: pour les mots de passe utilisés en wifi
#
def hex2mdp(ch):
    """hex2mdp(ch): convertit en texte clair les mots de passe donnés en chaîne hexadécimale"""
    if not pmain.estchaine(ch):
        raise ValueError ("erreur hex2mdp(ch): ch doit être une chaîne de caractères")
    if (len(ch)%2)!=0:
        raise ValueError ("erreur hex2mdp(ch): ch doit avoir un nombre pair de caractères")
    for car in ch:
        if not (car in "0123456789ABCDEFabcdef"):
            raise ValueError ("erreur hex2mdp(ch): la chaîne hexadécimale ch a des caractères anormaux")
    h=""
    for i in range(0,len(ch),2):
        h+=chr(int(ch[i:i+2],16))
    return h

###################################################################
# Convertit le nombre entier >=0 "n" en code de Gray
#
def dec2gray(n,nb=8):
    """dec2gray(n,nb=8): convertit le nombre entier >=0 "n" en code de Gray"""
    # Vérification des données
    if (not pmain.estentier(n)) or n<0:
        raise ValueError ("erreur dec2gray(n,nb=8):: n doit être un nombre entier positif ou nul")
    n=int(n)
    if (not pmain.estentier(nb)) or nb<=0:
        raise ValueError ("erreur dec2gray(n,nb=8):: nb doit être un nombre entier positif")
    nb=int(nb)
    # Calcul
    ch=""
    for k in xrange(nb,0,-1):
        ch+="01"[((n+(2**(k-1)))>>k)%2]
    return ch

###################################################################
# Convertit la chaîne binaire ch représentant un code de Gray, en nombre entier décimal
#
def gray2dec(ch):
    """gray2dec(ch): convertit la chaîne binaire ch représentant un code de Gray, en nombre entier décimal"""
    # Vérification des données
    if not pmain.estchaine(ch):
        raise ValueError ("erreur gray2dec(ch): ch doit être une chaîne de caractères")
    if not okbin(ch):
        raise ValueError ("erreur gray2dec(ch): mauvaise chaîne binaire ch")
    # Calcul
    i=ch.find("1")
    if i<0:
        return 0
    lg=len(ch)
    k=lg-i
    n=(2**k)-(2**(k-1))
    while k>1:
        k-=1
        b=int(ch[lg-k])
        y=0
        while (2*y+b)*(2**k)-(2**(k-1))+(2**k)-1 < n:
            y+=1
        np=(2*y+b)*(2**k)-(2**(k-1))
        if np>n:
            n=np
    return n

