#!/usr/bin/python
# -*- coding: utf-8 -*-

# attention: pour les accents, ce fichier doit être impérativement
#            lu et modifié sous éditeur avec encodage UTF-8

################################################################
######                                                    ######
######          Module additionnel: credit.py             ######
######                                                    ######
######            accompagnant la CALCULEXT               ######
######      [Calculatrice Extensible Version 1.40]        ######
######                                                    ######
################################################################

# Copyright (C) 2008 Jean-Paul Vidal dit "Tyrtamos" — Tous droits réservés.
#   Logiciel distribué sur le site <http://linux.jpvweb.com>
#
#   Ce programme est un logiciel libre. Vous pouvez le redistribuer ou le
#   modifier suivant les termes de la "GNU General Public License" version 3,
#   telle que publiée par la Free Software Foundation.
#
#   Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
#   AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
#   ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
#   Publique GNU pour plus de détails.
#
#   Ce programme doit être distribué avec une copie de la
#   License Générale Publique GNU (fichier license.html).
#   Si ce n’est pas le cas, consultez: <http://www.gnu.org/licenses/gpl.html>.

##################################################################
# Attention: configuration volontairement inhabituelle pour python
# les divisions entre 2 entiers seront décimales et pas entières!
# par exemple: 10/3 donnera "3.33333333333333" et pas "3"
# (les divisions entières seront désormais faites avec "//")
#
from __future__ import division

##################################################################
# Importation des modules système
#
import time

from sys import modules
pmain = modules['__main__']

##################################################################
# Importation des modules nécessaires aux fonctions du présent module
#
from math import *

############################################################
# Calcul des mensualités M d'un prêt de
# C à IA% d'intérêt par an, à rembouser en N mois
#
def menscredit(C,IA,N):
    """menscredit(C,IA,N): mensualités M d'un prêt C à IA% d'intérêt par an, à rembourser en N mois"""
    # Vérification des données
    if (not pmain.estnombre(C)) or (C<0):
        raise ValueError ("erreur menscredit(C,IA,N): le capital C doit être un nombre positif")
    if (not pmain.estnombre(IA)) or (IA<0):
        raise ValueError ("erreur menscredit(C,IA,N): l'intérêt annuel IA% doit être un nombre positif")
    if (not pmain.estnombre(N)) or (N<1):
        raise ValueError ("erreur menscredit(C,IA,N): le nombre de mois N doit être un nombre positif >1")
    # si le capital est nul, il n'y a rien à rembourser
    if C==0:
        M=0
        return M
    # si l'intérêt est nul, la mensualité ne dépend plus que de C et N
    if IA==0:
        M=C/N
        return M
    # calcul de la mensualité
    C=C
    I=IA/1200
    M=C*I*(1-1/(1-(1+I)**N))
    return M

############################################################
# Calcul du nombre de mois de remboursement d'un prêt
# connaissant le capital C, l'intérêt IA%/an et la mensualité M
#
def moiscredit(C,IA,M):
    """moiscredit(C,IA,M): nombre de mois de remboursement d'un prêt C ayant un intérêt IA%/an et une mensualité M"""
    # Verification des données
    if (not pmain.estnombre(C)) or (C<0):
        raise ValueError ("erreur moiscredit(C,IA,M): le capital C doit être un nombre positif ou nul")
    if (not pmain.estnombre(IA)) or (IA<0):
        raise ValueError ("erreur moiscredit(C,IA,M): l'intérêt annuel IA% doit être un nombre positif ou nul")
    if (not pmain.estnombre(M)) or (M<0):
        raise ValueError ("erreur moiscredit(C,IA,M): la mensualité M doit être un nombre positif")
    # si le capital est nul, il n'y a rien a rembourser
    if C==0:
        N=0
        return N
    # si l'intérêt est nul, la mensualité ne dépend que de N
    if IA==0:
        N=C/M
        return N
    # calcul du nombre de mois de remboursement
    C=C
    I=IA/1200
    N=log(1-1/(1-M/(C*I)))/log(1+I)
    return N

############################################################
# Calcul du capital emprunté à IA%/an d'intérêt
#  remboursé en N mois avec une mensualité de M
#
def capcredit(IA,M,N):
    """capcredit(IA,M,N): capital emprunté à IA%/an d'intérêt, remboursé en N mois avec une mensualité de M"""
    # Vérification des données
    if (not pmain.estnombre(IA)) or (IA<0):
        raise ValueError ("erreur capcredit(IA,M,N): l'intérêt annuel IA% doit être un nombre positif ou nul")
    if (not pmain.estnombre(M)) or (M<0):
        raise ValueError ("erreur capcredit(IA,M,N): la mensualité M doit être un nombre positif")
    if (not pmain.estnombre(N)) or (N<1):
        raise ValueError ("erreur capcredit(IA,M,N): le nombre de mois N doit être un nombre positif >1")
    # si l'intérêt est nul, le capital est facile à calculer!
    if IA==0:
        C=M*N
        return C
    # calcul du capital emprunté
    I=IA/1200
    M=M
    C=M/(I*(1-(1/(1-(1+I)**N))))
    return C

############################################################
# Calcul de l'intérêt annuel IA% connaissant le capital emprunté,
#    le nb de mois de remboursement et la mensualité
#
def intcredit(C,M,N):
    """intcredit(C,M,N): intérêt IA%/an, connaissant le capital emprunté C, la mensualité M et le nb de mois de remboursement N"""
    # Vérification des données
    if (not pmain.estnombre(C)) or (C<0):
        raise ValueError ("erreur intcredit(C,M,N): le capital C doit être un nombre positif ou nul")
    if (not pmain.estnombre(M)) or (M<=0):
        raise ValueError ("erreur intcredit(C,M,N): la mensualité M doit être un nombre positif")
    if (not pmain.estnombre(N)) or (N<1):
        raise ValueError ("erreur intcredit(C,M,N): le nombre de mois N doit être un nombre positif >1")
    if N*M<C:
        raise ValueError ("erreur intcredit(): données incorrectes C doit être >= N*M")
    # cas ou l'intérêt est manifestement nul
    if N*M==C:
        IA=0
        return IA
    # calcul de IA par recherche de zéro
    IA=10.0
    DI=1.0
    eps=0.000001
    F1=M-menscredit(C,IA,N)
    F2=F1
    while (fabs(F1)>eps):
        F1=F2
        IA=IA+DI
        F2=M-menscredit(C,IA,N)
        if ((F2*F1)<0) or (fabs(F2)>fabs(F1)):
            DI=-DI/2
    return IA

############################################################
# Calcul du coût d'un crédit connaissant le capital C,
#   la mensualité et le nb de mois de remboursement N
# option sup = coûts supplémentaires hors mensualité (ex: frais de dossier)
#
def coutcredit(C,M,N,sup=0):
    """coutcredit(C,M,N,sup=0): coût d'un crédit connaissant le capital C, la mensualité M et le nb de mois de remboursement N (sup = coût supplémentaire optionnel)"""
    # Vérification des données
    if (not pmain.estnombre(C)) or (C<0):
        raise ValueError ("erreur coutcredit(C,M,N,sup=0): le capital C doit être un nombre positif ou nul")
    if (not pmain.estnombre(M)) or (M<=0):
        raise ValueError ("erreur coutcredit(C,M,N,sup=0): la mensualité M doit être un nombre positif")
    if (not pmain.estnombre(N)) or (N<1):
        raise ValueError ("erreur coutcredit(C,M,N,sup=0): le nombre de mois N doit être un nombre positif >1")
    if N*M<C:
        raise ValueError ("erreur coutcredit(C,M,N,sup=0): données incorrectes C doit être >= N*M")
    # calcul du coût
    cout=M*N-C+sup
    return cout

